#!/usr/bin/perl
use strict;
use warnings;
use File::Basename;

my $preamble  = <<'END_PRE';
<!DOCTYPE html>
<!-- start header -->
<html>
<head>
	<title>XXPAGE_TITLEXX</title>
	<meta http-equiv="Content-Type" content="text/html; charset=UTF8">
	<link rel="stylesheet" href="../lesson.css">
	<script type="text/javascript" src="../lesson.js"></script>
</head>
<body id="artbody">
<div class="footer"><a href="../gtypist.html#about">About</a> | <a href="../gtypist.html#help">Help</a>
  | <a href="../gtypist.html">Online Typing Home</a>
  |<a href="http://hypatype.squte.com">HypaType Typing Accelerator</a></div>
<div class="promotion">Increase your typing speed, enter frequently typed text with fast shortcuts, and avoid spelling errors with
  <a href="http://hypatype.squte.com">HypaType</a> - the typing accelerator.</div>
<noscript>
  <h1 class="errMsg">Javascript needs to be enabled to use this lesson</h1>
</noscript>
<h1 id="title">XXPAGE_TITLEXX</h1>
<div id="instruction">Repeat each drill until you pass, then go on to the next. You will pass when your error rate is lower than three percent.</div>
<div id="lessons">
<!-- end header -->
END_PRE

my $postamble  = <<'END_POST';
<!-- start footer -->
</div>

<div id="source">View the lesson <a href="XXSOURCEXX">source file</a>.</div>
<div class="promotion">Increase your typing speed, enter boiler-plate text with fast shortcuts, and avoid spelling errors with
      <a href="http://hypatype.squte.com">HypaType</a>, the typing accelerator.</div>
<div class="footer"><a href="../gtypist.html#about">About</a> | <a href="../gtypist.html#help">Help</a>
  | <a href="../gtypist.html">Online Typing Home</a>
  |<a href="http://hypatype.squte.com">HypaType Typing Accelerator</a></div>
</body>
<!-- end footer -->
</html>
END_POST

my $infile = $ARGV[0];

my $outfile = $infile . ".html";
print "\nWriting file: $outfile.\n";
open (OUT, ">$outfile") or die "Can't open file: $!";

# Skip if this is not a file we can open.  Also skip backup
# files
die "Invalid file '$infile'" unless ((-f $infile) && (-r $infile) && ($infile !~ /~$/)) ;

# open file on command line
print "processing file $infile...\n";
open (IN, $infile) or die "Can't open file '$infile': $!";

# recursive descent would avoid having to have these globals
# are we currently processing a menu?
my $is_nav = 0;
# are we currently processing a drill?
my $in_drill= 0;
my $id = "";
my @tags;

# Loop through file
my $line  ;
# main contents to write to output. stored seperately so can put navigation at the top of the page
my $out = "";
# navigation (menu) is stored as the preamble needs to be updated
# before it's written
my $nav  = "";
my $test_count = 0;
# characters in this drill
my $char_count = 0;
my $last_B = "";

# drill or section title
my $title     = "";
# drill or section title length (unescaped)
my $titlelength = 0;

# get a unique id for the drill in case none is given with *:
my $uid_num = 0;
sub get_next_uid(){
	$uid_num++;
	return "GTYPE_ID_" . $uid_num;
}

# dont use lib so no dependencies
# not ideal, but isn't run on untrusted data
sub escape_html
{
  my $text = shift ;
  $text =~ s/&/&amp;/g ;
  $text =~ s/</&lt;/g ;
  $text =~ s/>/&gt;/g ;
	return $text;
}

sub end_drill()
{
  if($in_drill){
		$out .= "<table class=\"results\">\n";
		$out .= "\t<tr><td>Total Characters</td><td id=tot_$id>$char_count</td></tr>\n";
		$out .= "\t<tr><td>Errors</td><td id=errs_$id>-</td></tr>\n";
		$out .= "\t<tr><td>Error Rate</td><td id=perErrs_$id>-</td></tr>\n";
		$out .= "\t<tr><td>WPM</td><td id=wpm_$id>-</td></tr>\n";
		$out .= "\t<tr><td colspan=2><div id=status_$id class=\"incompleteRes\">Incomplete</td></tr>\n";
		$out .= "</table>\n";
		$out .= "<button type=\"button\" id=start_$id onclick=\"startDrill(this)\" >Start</button> \n";
	}
  $in_drill = 0;

  while (@tags){
    my $tagline = pop(@tags) ;
    if($is_nav) {
      $nav .= "$tagline\n";
    }else{
      $out .= "$tagline\n";
    }
  }
  $is_nav = 0;
  $char_count = 0;
  $last_B = "";
}

sub drill_line()
{
  my $cnt_id = $id . "_" . $test_count;

  $out .= "<span id=text_$cnt_id>$title</span>\n";
  $char_count += $titlelength;
  $out .= "<input id=input_$cnt_id size=\"$titlelength\" maxlength=\"$titlelength\" oninput=\"keyInput(this)\" onkeypress=keypress(event) disabled" .
             " autocomplete=\"off\" autocorrect=\"off\" autocapitalize=\"off\" spellcheck=\"false\"/>\n";
}

foreach $line (<IN>)
{
# remove comments and end whitespace
	my $stmt = $line ;

	$stmt =~ s/#.*//;
	$stmt =~ s/\s*$//;

	if (!$stmt){
    # blank line = end of section
    end_drill();
		next;
	}

# ?: where '?' != a space tells us type of section
	my $typ  = $stmt  ;
	# split into before ':' and after
	$typ  =~ s/\:.*//;
	$title = $stmt   ;
	$title =~ s/.\://;
	$title =~ s/^\s*//;
# get length before escaping
  $titlelength = length($title);
	$title = escape_html($title);
	if($typ eq 'G'){
# link to other section - done using css
	}

	if($typ eq '*'){
# can get sequential drills without a blank line
		if($in_drill){
			end_drill;
		}
		$id=$title;
	}

	if($typ eq 'B'){
# can get sequential s without a blank line
		if($in_drill){
			end_drill;
		}
		$out .= "<h2 id=$id>$title</h2>\n";
		$id="" ;
		$last_B = $title;
	}

	if($typ eq 'I'){
		$out .= "<h3>$title</h3>\n";
	}

	if($typ eq 'T'){
		$out .= "<h4>$title</h4>\n<span>\n";
		push (@tags, "</span>\n");
	}

  # 'S' is a speed test, treat the same as a drill for now
	# lower case d,s means ignore errors, ignored for now
	if( ($typ eq 'D') || ($typ eq 'S') || ($typ eq 'd') || ($typ eq 's') ){
		# can get sequential s without a blank line
    if($in_drill){
      end_drill;
     }
    # drill start
		# sometimes get a drill without a *: - so make own id		#
		if($id eq ""){
			$id =  get_next_uid();
		}
    $out .= "<div id=drill_$id class=\"drill\">\n";

    push (@tags, "</div>\n");

    $test_count = 0;
    $char_count = 0;
    drill_line;
		# $id="" ;
    $in_drill = 1;
	}

	if($typ eq ' '){
		if($is_nav){
			my $rest = $title;
			# in form $ :_C_S_C3  "Lesson C3     A and O"$
			$rest  =~ s/[^ ]+ +"//;
			$rest =~ s/"$//;
			my $link = $title ;
			$link =~ s/ +".*//;
			$nav .= "<a href=\"#$link\">$rest</a>\n";
		}else{
      if($in_drill){
        $test_count++;
        drill_line;
      }else{
				if($title eq ""){
					$out .= "<br\>\n";
				}else{
					$out .= "$title\n";
				}
      }
		}
	}

	if($typ eq 'M'){
		my $rest = $title;
		# in form $M: UP=_EXIT "The C series contains the following 17 lesson$
		$rest  =~ s/[^ ]+ +"//;
		$rest  =~ s/ *"//;
		$rest =~ s/"$//;
		$nav .= "<nav><h1>$rest</h1>\n";
		push (@tags, "</nav>\n");
		$is_nav = 1;
    # where you have a 'B:' before an 'M:', that is the page title, else use the menu title
    my $page_title;
    if($last_B){
      $page_title = $last_B;
    }else{
      $page_title = $rest;
    }
#    in case of multiple menus, use first found
    $preamble =~ s/XXPAGE_TITLEXX/$page_title/g;
	}

	if($stmt eq ':'){
		$out .= "<br>\n";
	}
}

close (IN);

end_drill;

my $filename = basename($infile);
$postamble =~ s/XXSOURCEXX/$filename/g;

print OUT "$preamble";

print OUT "$nav";
print OUT "$out";

print OUT "$postamble";
close (OUT);


